#include <stdlib.h>
#include <stdio.h>
#include "gamePlayers.h"

char * makeNewName(int gamesPlayed) {
  int m = gamesPlayed;
  int digits = 1;
  while((m /= 10) > 0) ++digits;
  char * ans = (char*)malloc((6+digits+1)*sizeof(char));
  snprintf(ans,6+digits+1,"player%d",gamesPlayed);
  return ans;
}

// functions for how to play
int always_play(void* my_state, play_info_t play, int num) {
  return 1;
}
int guaranteed_play(void* my_state, play_info_t play, int num) {
  return num==1 || num==100 || (play.numAfter == 0 && play.numPlaying == 0);
}
int noothers_play(void* my_state, play_info_t play, int num) {
  return play.numPlaying == 0;
}
int decrement_to_zero(void* my_state, play_info_t play, int num) {
  int* counter = (int*)my_state;
  if(*counter == 0)
    return 0;
  *counter -= 1;
  return 1; 
}

// functions for how to choose
int always_low(void* my_state, int numPlaying, int num) {
  return 0;
}
int always_high(void* my_state, int numPlaying, int num) {
  return 1;
}
int guaranteed_choose(void* my_state, int numPlaying, int num) {
  return num==100;
}
int closer_choose(void* my_state, int numPlaying, int num) {
  return num > 50;
}

// functions for how to destruct
void ignore_pointer(void* my_state) { }
void free_int(void* my_state) {
  free(my_state);
}

player_t* new_player(int gamesPlayed) {
  player_t* ans = (player_t*)malloc(sizeof(player_t));
  ans->name = makeNewName(gamesPlayed);
  switch(gamesPlayed % 5) {
  case 1:
    // when plays, always wins (but doesn't play enough)
    ans->player_state     = NULL;
    ans->play_function    = guaranteed_play;
    ans->choose_function  = guaranteed_choose;
    ans->state_destructor = ignore_pointer;
    break;
  case 2:
    // plays whenever first to play, makes reasonable choice
    ans->player_state     = NULL;
    ans->play_function    = noothers_play;
    ans->choose_function  = closer_choose;
    ans->state_destructor = ignore_pointer;
    break;
  case 3:
    // always play; makes reasonable choice
    ans->player_state     = NULL;
    ans->play_function    = always_play;
    ans->choose_function  = closer_choose;
    ans->state_destructor = ignore_pointer;
    break;
  case 4:
    // plays low for 10 hands then stops playing anything
    { int * p = (int*)malloc(sizeof(int));
    *p = 10;
    ans->player_state     = p;
    ans->play_function    = decrement_to_zero;
    ans->choose_function  = always_low;
    ans->state_destructor = free_int;
    }
    break;
  default: 
    // always play; always choose high (stupid strategy)
    ans->player_state     = NULL;
    ans->play_function    = always_play;
    ans->choose_function  = always_high;
    ans->state_destructor = ignore_pointer;
  }
  return ans;
}
